@@ -1006,6 +1006,8 @@ static gboolean process_changes(gpointer user_data)
 	if (data->removed != NULL)
 		emit_interfaces_removed(data);
 
+	data->process_id = 0;
+
 	return FALSE;
 }
 
@@ -1019,6 +1021,7 @@ static void generic_unregister(DBusConnection *connection, void *user_data)
 
 	if (data->process_id > 0) {
 		g_source_remove(data->process_id);
+		data->process_id = 0;
 		process_changes(data);
 	}
 
@@ -1510,11 +1513,20 @@ gboolean g_dbus_send_message_with_reply(DBusConnection *connection,
 					DBusMessage *message,
 					DBusPendingCall **call, int timeout)
 {
+	dbus_bool_t ret;
+
 	/* Flush pending signal to guarantee message order */
 	g_dbus_flush(connection);
 
-	return dbus_connection_send_with_reply(connection, message, call,
+	ret = dbus_connection_send_with_reply(connection, message, call,
 								timeout);
+
+	if (ret == TRUE && call != NULL && *call == NULL) {
+		error("Unable to send message (passing fd blocked?)");
+		return FALSE;
+	}
+
+	return ret;
 }
 
 gboolean g_dbus_send_error_valist(DBusConnection *connection,
