@@ -170,12 +170,9 @@ uint8_t *dhcpv6_get_option(struct dhcpv6_packet *packet, uint16_t pkt_len,
 			break;
 
 		if (opt_code == code) {
-			if (option_len != NULL)
+			if (option_len)
 				*option_len = opt_len;
-			if (rem < 0)
-				goto bad_packet;
-			else
-				found = optionptr + 2 + 2;
+			found = optionptr + 2 + 2;
 			count++;
 		}
 
@@ -185,15 +182,15 @@ uint8_t *dhcpv6_get_option(struct dhcpv6_packet *packet, uint16_t pkt_len,
 		optionptr += len;
 	}
 
-	if (option_count != NULL)
+	if (option_count)
 		*option_count = count;
 
 	return found;
 
 bad_packet:
-	if (option_len != NULL)
+	if (option_len)
 		*option_len = 0;
-	if (option_count != NULL)
+	if (option_count)
 		*option_count = 0;
 	return NULL;
 }
@@ -371,17 +368,17 @@ void dhcpv6_init_header(struct dhcpv6_packet *packet, uint8_t type)
 	packet->transaction_id[2] = id & 0xff;
 }
 
-static gboolean check_vendor(uint8_t  *option_vendor, const char *vendor)
+static bool check_vendor(uint8_t  *option_vendor, const char *vendor)
 {
 	uint8_t vendor_length = sizeof(vendor) - 1;
 
 	if (option_vendor[OPT_LEN - OPT_DATA] != vendor_length)
-		return FALSE;
+		return false;
 
 	if (memcmp(option_vendor, vendor, vendor_length) != 0)
-		return FALSE;
+		return false;
 
-	return TRUE;
+	return true;
 }
 
 static void check_broken_vendor(struct dhcp_packet *packet)
@@ -392,10 +389,10 @@ static void check_broken_vendor(struct dhcp_packet *packet)
 		return;
 
 	vendor = dhcp_get_option(packet, DHCP_VENDOR);
-	if (vendor == NULL)
+	if (!vendor)
 		return;
 
-	if (check_vendor(vendor, "MSFT 98") == TRUE)
+	if (check_vendor(vendor, "MSFT 98"))
 		packet->flags |= htons(BROADCAST_FLAG);
 }
 
@@ -496,7 +493,7 @@ int dhcpv6_send_packet(int index, struct dhcpv6_packet *dhcp_pkt, int len)
 
 	control_buf_len = CMSG_SPACE(sizeof(struct in6_pktinfo));
 	control_buf = g_try_malloc0(control_buf_len);
-	if (control_buf == NULL) {
+	if (!control_buf) {
 		close(fd);
 		return -ENOMEM;
 	}
@@ -727,16 +724,16 @@ char *get_interface_name(int index)
 	return g_strdup(ifr.ifr_name);
 }
 
-gboolean interface_is_up(int index)
+bool interface_is_up(int index)
 {
 	int sk, err;
 	struct ifreq ifr;
-	gboolean ret = FALSE;
+	bool ret = false;
 
 	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	if (sk < 0) {
 		perror("Open socket error");
-		return FALSE;
+		return false;
 	}
 
 	memset(&ifr, 0, sizeof(ifr));
@@ -755,7 +752,7 @@ gboolean interface_is_up(int index)
 	}
 
 	if (ifr.ifr_flags & IFF_UP)
-		ret = TRUE;
+		ret = true;
 
 done:
 	close(sk);
