@@ -138,15 +138,6 @@ struct _GSupplicantSSID {
 
 typedef struct _GSupplicantSSID GSupplicantSSID;
 
-/*
- * Max number of SSIDs that can be scanned.
- * In wpa_s 0.7x the limit is 4.
- * In wps_s 0.8 or later it is 16.
- * The value is only used if wpa_supplicant does not return any max limit
- * for number of scannable SSIDs.
- */
-#define WPAS_MAX_SCAN_SSIDS 4
-
 struct scan_ssid {
 	unsigned char ssid[32];
 	uint8_t ssid_len;
@@ -157,6 +148,7 @@ struct _GSupplicantScanParams {
 
 	uint8_t num_ssids;
 
+	uint8_t num_freqs;
 	uint16_t *freqs;
 };
 
