@@ -2,7 +2,7 @@
  *
  *  Resolver library with GLib integration
  *
- *  Copyright (C) 2009-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2009-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -34,6 +34,7 @@
 #include <netdb.h>
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
+#include <net/if.h>
 
 #include "gresolv.h"
 
@@ -43,7 +44,7 @@ struct sort_result {
 	int dst_scope;
 	int src_label;
 	int dst_label;
-	gboolean reachable;
+	bool reachable;
 	union {
 		struct sockaddr sa;
 		struct sockaddr_in sin;
@@ -78,6 +79,7 @@ struct resolv_lookup {
 struct resolv_query {
 	GResolv *resolv;
 
+	int nr_ns;
 	guint timeout;
 
 	uint16_t msgid;
@@ -97,7 +99,9 @@ struct resolv_nameserver {
 };
 
 struct _GResolv {
-	gint ref_count;
+	int ref_count;
+
+	int result_family;
 
 	guint next_lookup_id;
 	GQueue *lookup_queue;
@@ -112,24 +116,34 @@ struct _GResolv {
 	gpointer debug_data;
 };
 
-static inline void debug(GResolv *resolv, const char *format, ...)
+#define debug(resolv, format, arg...)				\
+	_debug(resolv, __FILE__, __func__, format, ## arg)
+
+static void _debug(GResolv *resolv, const char *file, const char *caller,
+						const char *format, ...)
 {
 	char str[256];
 	va_list ap;
+	int len;
 
-	if (resolv->debug_func == NULL)
+	if (!resolv->debug_func)
 		return;
 
 	va_start(ap, format);
 
-	if (vsnprintf(str, sizeof(str), format, ap) > 0)
-		resolv->debug_func(str, resolv->debug_data);
+	if ((len = snprintf(str, sizeof(str), "%s:%s() resolv %p ",
+						file, caller, resolv)) > 0) {
+		if (vsnprintf(str + len, sizeof(str) - len, format, ap) > 0)
+			resolv->debug_func(str, resolv->debug_data);
+	}
 
 	va_end(ap);
 }
 
 static void destroy_query(struct resolv_query *query)
 {
+	debug(query->resolv, "query %p timeout %d", query, query->timeout);
+
 	if (query->timeout > 0)
 		g_source_remove(query->timeout);
 
@@ -138,13 +152,16 @@ static void destroy_query(struct resolv_query *query)
 
 static void destroy_lookup(struct resolv_lookup *lookup)
 {
-	if (lookup->ipv4_query != NULL) {
+	debug(lookup->resolv, "lookup %p id %d ipv4 %p ipv6 %p",
+		lookup, lookup->id, lookup->ipv4_query, lookup->ipv6_query);
+
+	if (lookup->ipv4_query) {
 		g_queue_remove(lookup->resolv->query_queue,
 						lookup->ipv4_query);
 		destroy_query(lookup->ipv4_query);
 	}
 
-	if (lookup->ipv6_query != NULL) {
+	if (lookup->ipv6_query) {
 		g_queue_remove(lookup->resolv->query_queue,
 						lookup->ipv6_query);
 		destroy_query(lookup->ipv6_query);
@@ -159,7 +176,8 @@ static void find_srcaddr(struct sort_result *res)
 	socklen_t sl = sizeof(res->src);
 	int fd;
 
-	fd = socket(res->dst.sa.sa_family, SOCK_DGRAM, IPPROTO_IP);
+	fd = socket(res->dst.sa.sa_family, SOCK_DGRAM | SOCK_CLOEXEC,
+			IPPROTO_IP);
 	if (fd < 0)
 		return;
 
@@ -173,7 +191,7 @@ static void find_srcaddr(struct sort_result *res)
 		return;
 	}
 
-	res->reachable = TRUE;
+	res->reachable = true;
 	close(fd);
 }
 
@@ -262,21 +280,21 @@ static const struct gai_table gai_precedences[] = {
 static unsigned char v4mapped[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 				    0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00 };
 
-static gboolean mask_compare(const unsigned char *one,
+static bool mask_compare(const unsigned char *one,
 					const unsigned char *two, int mask)
 {
 	if (mask > 8) {
 		if (memcmp(one, two, mask / 8))
-			return FALSE;
+			return false;
 		one += mask / 8;
 		two += mask / 8;
 		mask %= 8;
 	}
 
 	if (mask && ((*one ^ *two) >> (8 - mask)))
-		return FALSE;
+		return false;
 
-	return TRUE;
+	return true;
 }
 
 static int match_gai_table(struct sockaddr *sa, const struct gai_table *tbl)
@@ -451,45 +469,58 @@ static void rfc3484_sort_results(struct resolv_lookup *lookup)
 
 static void sort_and_return_results(struct resolv_lookup *lookup)
 {
-	char buf[100];
+	char buf[INET6_ADDRSTRLEN + 1];
 	GResolvResultStatus status;
 	char **results = g_try_new0(char *, lookup->nr_results + 1);
+	GResolvResultFunc result_func = lookup->result_func;
+	void *result_data = lookup->result_data;
 	int i, n = 0;
 
 	if (!results)
 		return;
 
+	memset(buf, 0, INET6_ADDRSTRLEN + 1);
+
 	rfc3484_sort_results(lookup);
 
 	for (i = 0; i < lookup->nr_results; i++) {
 		if (lookup->results[i].dst.sa.sa_family == AF_INET) {
-			if (inet_ntop(AF_INET,
+			if (!inet_ntop(AF_INET,
 					&lookup->results[i].dst.sin.sin_addr,
-					buf, sizeof(buf)) == NULL)
+					buf, sizeof(buf) - 1))
 				continue;
 		} else if (lookup->results[i].dst.sa.sa_family == AF_INET6) {
-			if (inet_ntop(AF_INET6,
+			if (!inet_ntop(AF_INET6,
 					&lookup->results[i].dst.sin6.sin6_addr,
-					buf, sizeof(buf)) == NULL)
+					buf, sizeof(buf) - 1))
 				continue;
 		} else
 			continue;
 
-		results[n++] = strdup(buf);
+		results[n++] = g_strdup(buf);
 	}
 
 	results[n++] = NULL;
 
-	status = lookup->ipv4_status;
-
-	if (status == G_RESOLV_RESULT_STATUS_SUCCESS)
+	if (lookup->resolv->result_family == AF_INET)
+		status = lookup->ipv4_status;
+	else if (lookup->resolv->result_family == AF_INET6)
 		status = lookup->ipv6_status;
+	else {
+		if (lookup->ipv6_status == G_RESOLV_RESULT_STATUS_SUCCESS)
+			status = lookup->ipv6_status;
+		else
+			status = lookup->ipv4_status;
+	}
 
-	lookup->result_func(status, results, lookup->result_data);
+	debug(lookup->resolv, "lookup %p received %d results", lookup, n);
 
-	g_strfreev(results);
 	g_queue_remove(lookup->resolv->lookup_queue, lookup);
 	destroy_lookup(lookup);
+
+	result_func(status, results, result_data);
+
+	g_strfreev(results);
 }
 
 static gboolean query_timeout(gpointer user_data)
@@ -508,25 +539,27 @@ static gboolean query_timeout(gpointer user_data)
 		lookup->ipv6_query = NULL;
 	}
 
-	if (lookup->ipv4_query == NULL && lookup->ipv4_query == NULL)
-		sort_and_return_results(lookup);
-
-	destroy_query(query);
 	g_queue_remove(resolv->query_queue, query);
+	destroy_query(query);
+
+	if (!lookup->ipv4_query && !lookup->ipv6_query)
+		sort_and_return_results(lookup);
 
 	return FALSE;
 }
 
 static void free_nameserver(struct resolv_nameserver *nameserver)
 {
-	if (nameserver == NULL)
+	if (!nameserver)
 		return;
 
 	if (nameserver->udp_watch > 0)
 		g_source_remove(nameserver->udp_watch);
 
-	if (nameserver->udp_channel != NULL)
+	if (nameserver->udp_channel) {
+		g_io_channel_shutdown(nameserver->udp_channel, TRUE, NULL);
 		g_io_channel_unref(nameserver->udp_channel);
+	}
 
 	g_free(nameserver->address);
 	g_free(nameserver);
@@ -547,24 +580,27 @@ static void flush_nameservers(GResolv *resolv)
 static int send_query(GResolv *resolv, const unsigned char *buf, int len)
 {
 	GList *list;
+	int nr_ns;
 
-	if (resolv->nameserver_list == NULL)
+	if (!resolv->nameserver_list)
 		return -ENOENT;
 
-	for (list = g_list_first(resolv->nameserver_list);
-					list; list = g_list_next(list)) {
+	for (list = g_list_first(resolv->nameserver_list), nr_ns = 0;
+				list; list = g_list_next(list), nr_ns++) {
 		struct resolv_nameserver *nameserver = list->data;
 		int sk, sent;
 
-		if (nameserver->udp_channel == NULL)
+		if (!nameserver->udp_channel)
 			continue;
 
 		sk = g_io_channel_unix_get_fd(nameserver->udp_channel);
 
 		sent = send(sk, buf, len, 0);
+		if (sent < 0)
+			continue;
 	}
 
-	return 0;
+	return nr_ns;
 }
 
 static gint compare_lookup_id(gconstpointer a, gconstpointer b)
@@ -599,8 +635,10 @@ static void add_result(struct resolv_lookup *lookup, int family,
 							const void *data)
 {
 	int n = lookup->nr_results++;
-	lookup->results = g_realloc(lookup->results,
+	lookup->results = g_try_realloc(lookup->results,
 					sizeof(struct sort_result) * (n + 1));
+	if (!lookup->results)
+		return;
 
 	memset(&lookup->results[n], 0, sizeof(struct sort_result));
 
@@ -623,11 +661,17 @@ static void parse_response(struct resolv_nameserver *nameserver,
 	GList *list;
 	ns_msg msg;
 	ns_rr rr;
-	int i, n, rcode, count;
+	int i, rcode, count;
 
 	debug(resolv, "response from %s", nameserver->address);
 
-	ns_initparse(buf, len, &msg);
+	if (ns_initparse(buf, len, &msg) < 0)
+		return;
+
+	list = g_queue_find_custom(resolv->query_queue,
+			GUINT_TO_POINTER(ns_msg_id(msg)), compare_query_msgid);
+	if (!list)
+		return;
 
 	rcode = ns_msg_getflag(msg, ns_f_rcode);
 	count = ns_msg_count(msg, ns_s_an);
@@ -636,22 +680,22 @@ static void parse_response(struct resolv_nameserver *nameserver,
 					ns_msg_id(msg), rcode, count);
 
 	switch (rcode) {
-	case 0:
+	case ns_r_noerror:
 		status = G_RESOLV_RESULT_STATUS_SUCCESS;
 		break;
-	case 1:
+	case ns_r_formerr:
 		status = G_RESOLV_RESULT_STATUS_FORMAT_ERROR;
 		break;
-	case 2:
+	case ns_r_servfail:
 		status = G_RESOLV_RESULT_STATUS_SERVER_FAILURE;
 		break;
-	case 3:
+	case ns_r_nxdomain:
 		status = G_RESOLV_RESULT_STATUS_NAME_ERROR;
 		break;
-	case 4:
+	case ns_r_notimpl:
 		status = G_RESOLV_RESULT_STATUS_NOT_IMPLEMENTED;
 		break;
-	case 5:
+	case ns_r_refused:
 		status = G_RESOLV_RESULT_STATUS_REFUSED;
 		break;
 	default:
@@ -659,24 +703,19 @@ static void parse_response(struct resolv_nameserver *nameserver,
 		break;
 	}
 
-	list = g_queue_find_custom(resolv->query_queue,
-			GUINT_TO_POINTER(ns_msg_id(msg)), compare_query_msgid);
-	if (!list)
-		return;
-
 	query = list->data;
+	query->nr_ns--;
+
 	lookup = query->lookup;
 
-	if (query == lookup->ipv6_query) {
+	if (query == lookup->ipv6_query)
 		lookup->ipv6_status = status;
-		lookup->ipv6_query = NULL;
-	} else if (query == lookup->ipv4_query) {
+	else if (query == lookup->ipv4_query)
 		lookup->ipv4_status = status;
-		lookup->ipv4_query = NULL;
-	}
 
-	for (i = 0, n = 0; i < count; i++) {
-		ns_parserr(&msg, ns_s_an, i, &rr);
+	for (i = 0; i < count; i++) {
+		if (ns_parserr(&msg, ns_s_an, i, &rr) < 0)
+			continue;
 
 		if (ns_rr_class(rr) != ns_c_in)
 			continue;
@@ -693,11 +732,19 @@ static void parse_response(struct resolv_nameserver *nameserver,
 		}
 	}
 
-	if (lookup->ipv4_query == NULL && lookup->ipv6_query == NULL)
-		sort_and_return_results(lookup);
+	if (status != G_RESOLV_RESULT_STATUS_SUCCESS && query->nr_ns > 0)
+		return;
+
+	if (query == lookup->ipv6_query)
+		lookup->ipv6_query = NULL;
+	else
+		lookup->ipv4_query = NULL;
 
-	destroy_query(query);
 	g_queue_remove(resolv->query_queue, query);
+	destroy_query(query);
+
+	if (!lookup->ipv4_query && !lookup->ipv6_query)
+		sort_and_return_results(lookup);
 }
 
 static gboolean received_udp_data(GIOChannel *channel, GIOCondition cond,
@@ -753,6 +800,25 @@ static int connect_udp_channel(struct resolv_nameserver *nameserver)
 		return -EIO;
 	}
 
+	/*
+	 * If nameserver points to localhost ip, their is no need to
+	 * bind the socket on any interface.
+	 */
+	if (nameserver->resolv->index > 0 &&
+			strncmp(nameserver->address, "127.0.0.1", 9) != 0) {
+		char interface[IF_NAMESIZE];
+
+		memset(interface, 0, IF_NAMESIZE);
+		if (if_indextoname(nameserver->resolv->index, interface)) {
+			if (setsockopt(sk, SOL_SOCKET, SO_BINDTODEVICE,
+						interface, IF_NAMESIZE) < 0) {
+				close(sk);
+				freeaddrinfo(rp);
+				return -EIO;
+			}
+		}
+	}
+
 	if (connect(sk, rp->ai_addr, rp->ai_addrlen) < 0) {
 		close(sk);
 		freeaddrinfo(rp);
@@ -762,7 +828,7 @@ static int connect_udp_channel(struct resolv_nameserver *nameserver)
 	freeaddrinfo(rp);
 
 	nameserver->udp_channel = g_io_channel_unix_new(sk);
-	if (nameserver->udp_channel == NULL) {
+	if (!nameserver->udp_channel) {
 		close(sk);
 		return -ENOMEM;
 	}
@@ -784,21 +850,23 @@ GResolv *g_resolv_new(int index)
 		return NULL;
 
 	resolv = g_try_new0(GResolv, 1);
-	if (resolv == NULL)
+	if (!resolv)
 		return NULL;
 
 	resolv->ref_count = 1;
 
+	resolv->result_family = AF_UNSPEC;
+
 	resolv->next_lookup_id = 1;
 
 	resolv->query_queue = g_queue_new();
-	if (resolv->query_queue == NULL) {
+	if (!resolv->query_queue) {
 		g_free(resolv);
 		return NULL;
 	}
 
 	resolv->lookup_queue = g_queue_new();
-	if (resolv->lookup_queue == NULL) {
+	if (!resolv->lookup_queue) {
 		g_queue_free(resolv->query_queue);
 		g_free(resolv);
 		return NULL;
@@ -814,10 +882,10 @@ GResolv *g_resolv_new(int index)
 
 GResolv *g_resolv_ref(GResolv *resolv)
 {
-	if (resolv == NULL)
+	if (!resolv)
 		return NULL;
 
-	g_atomic_int_inc(&resolv->ref_count);
+	__sync_fetch_and_add(&resolv->ref_count, 1);
 
 	return resolv;
 }
@@ -825,15 +893,23 @@ GResolv *g_resolv_ref(GResolv *resolv)
 void g_resolv_unref(GResolv *resolv)
 {
 	struct resolv_query *query;
+	struct resolv_lookup *lookup;
 
-	if (resolv == NULL)
+	if (!resolv)
 		return;
 
-	if (g_atomic_int_dec_and_test(&resolv->ref_count) == FALSE)
+	if (__sync_fetch_and_sub(&resolv->ref_count, 1) != 1)
 		return;
 
-	while ((query = g_queue_pop_head(resolv->query_queue)))
+	while ((lookup = g_queue_pop_head(resolv->lookup_queue))) {
+		debug(resolv, "lookup %p id %d", lookup, lookup->id);
+		destroy_lookup(lookup);
+	}
+
+	while ((query = g_queue_pop_head(resolv->query_queue))) {
+		debug(resolv, "query %p", query);
 		destroy_query(query);
+	}
 
 	g_queue_free(resolv->query_queue);
 	g_queue_free(resolv->lookup_queue);
@@ -848,47 +924,46 @@ void g_resolv_unref(GResolv *resolv)
 void g_resolv_set_debug(GResolv *resolv, GResolvDebugFunc func,
 						gpointer user_data)
 {
-	if (resolv == NULL)
+	if (!resolv)
 		return;
 
 	resolv->debug_func = func;
 	resolv->debug_data = user_data;
 }
 
-gboolean g_resolv_add_nameserver(GResolv *resolv, const char *address,
+bool g_resolv_add_nameserver(GResolv *resolv, const char *address,
 					uint16_t port, unsigned long flags)
 {
 	struct resolv_nameserver *nameserver;
 
-	if (resolv == NULL)
-		return FALSE;
+	if (!resolv)
+		return false;
 
 	nameserver = g_try_new0(struct resolv_nameserver, 1);
-	if (nameserver == NULL)
-		return FALSE;
+	if (!nameserver)
+		return false;
 
 	nameserver->address = g_strdup(address);
 	nameserver->port = port;
 	nameserver->flags = flags;
+	nameserver->resolv = resolv;
 
 	if (connect_udp_channel(nameserver) < 0) {
 		free_nameserver(nameserver);
-		return FALSE;
+		return false;
 	}
 
-	nameserver->resolv = resolv;
-
 	resolv->nameserver_list = g_list_append(resolv->nameserver_list,
 								nameserver);
 
 	debug(resolv, "setting nameserver %s", address);
 
-	return TRUE;
+	return true;
 }
 
 void g_resolv_flush_nameservers(GResolv *resolv)
 {
-	if (resolv == NULL)
+	if (!resolv)
 		return;
 
 	flush_nameservers(resolv);
@@ -900,7 +975,7 @@ static gint add_query(struct resolv_lookup *lookup, const char *hostname, int ty
 	unsigned char buf[4096];
 	int len;
 
-	if (query == NULL)
+	if (!query)
 		return -ENOMEM;
 
 	len = res_mkquery(ns_o_query, hostname, ns_c_in, type,
@@ -908,7 +983,10 @@ static gint add_query(struct resolv_lookup *lookup, const char *hostname, int ty
 
 	query->msgid = buf[0] << 8 | buf[1];
 
-	if (send_query(lookup->resolv, buf, len) < 0) {
+	debug(lookup->resolv, "sending %d bytes", len);
+
+	query->nr_ns = send_query(lookup->resolv, buf, len);
+	if (query->nr_ns <= 0) {
 		g_free(query);
 		return -EIO;
 	}
@@ -918,6 +996,9 @@ static gint add_query(struct resolv_lookup *lookup, const char *hostname, int ty
 
 	g_queue_push_tail(lookup->resolv->query_queue, query);
 
+	debug(lookup->resolv, "lookup %p id %d query %p", lookup, lookup->id,
+									query);
+
 	query->timeout = g_timeout_add_seconds(5, query_timeout, query);
 
 	if (type == ns_t_aaaa)
@@ -933,12 +1014,12 @@ guint g_resolv_lookup_hostname(GResolv *resolv, const char *hostname,
 {
 	struct resolv_lookup *lookup;
 
-	debug(resolv, "lookup hostname %s", hostname);
-
-	if (resolv == NULL)
+	if (!resolv)
 		return 0;
 
-	if (resolv->nameserver_list == NULL) {
+	debug(resolv, "hostname %s", hostname);
+
+	if (!resolv->nameserver_list) {
 		int i;
 
 		for (i = 0; i < resolv->res.nscount; i++) {
@@ -959,12 +1040,12 @@ guint g_resolv_lookup_hostname(GResolv *resolv, const char *hostname,
 				g_resolv_add_nameserver(resolv, buf, 53, 0);
 		}
 
-		if (resolv->nameserver_list == NULL)
+		if (!resolv->nameserver_list)
 			g_resolv_add_nameserver(resolv, "127.0.0.1", 53, 0);
 	}
 
 	lookup = g_try_new0(struct resolv_lookup, 1);
-	if (lookup == NULL)
+	if (!lookup)
 		return 0;
 
 	lookup->resolv = resolv;
@@ -972,34 +1053,65 @@ guint g_resolv_lookup_hostname(GResolv *resolv, const char *hostname,
 	lookup->result_data = user_data;
 	lookup->id = resolv->next_lookup_id++;
 
-	if (add_query(lookup, hostname, ns_t_a)) {
-		g_free(lookup);
-		return -EIO;
+	if (resolv->result_family != AF_INET6) {
+		if (add_query(lookup, hostname, ns_t_a)) {
+			g_free(lookup);
+			return -EIO;
+		}
 	}
 
-	if (add_query(lookup, hostname, ns_t_aaaa)) {
-		destroy_query(lookup->ipv4_query);
-		g_queue_remove(resolv->query_queue, lookup->ipv4_query);
-		g_free(lookup);
-		return -EIO;
+	if (resolv->result_family != AF_INET) {
+		if (add_query(lookup, hostname, ns_t_aaaa)) {
+			if (resolv->result_family != AF_INET6) {
+				g_queue_remove(resolv->query_queue,
+						lookup->ipv4_query);
+				destroy_query(lookup->ipv4_query);
+			}
+
+			g_free(lookup);
+			return -EIO;
+		}
 	}
 
 	g_queue_push_tail(resolv->lookup_queue, lookup);
+
+	debug(resolv, "lookup %p id %d", lookup, lookup->id);
+
 	return lookup->id;
 }
 
-gboolean g_resolv_cancel_lookup(GResolv *resolv, guint id)
+bool g_resolv_cancel_lookup(GResolv *resolv, guint id)
 {
+	struct resolv_lookup *lookup;
 	GList *list;
 
+	debug(resolv, "lookup id %d", id);
+
 	list = g_queue_find_custom(resolv->lookup_queue,
 				GUINT_TO_POINTER(id), compare_lookup_id);
 
-	if (list == NULL)
-		return FALSE;
+	if (!list)
+		return false;
 
-	destroy_lookup(list->data);
-	g_queue_remove(resolv->query_queue, list->data);
+	lookup = list->data;
 
-	return TRUE;
+	debug(resolv, "lookup %p", lookup);
+
+	g_queue_remove(resolv->lookup_queue, lookup);
+	destroy_lookup(lookup);
+
+	return true;
+}
+
+bool g_resolv_set_address_family(GResolv *resolv, int family)
+{
+	if (!resolv)
+		return false;
+
+	if (family != AF_UNSPEC && family != AF_INET && family != AF_INET6)
+		return false;
+
+	resolv->result_family = family;
+
+	return true;
 }
