@@ -22,7 +22,7 @@
 #ifndef __CONNMAN_SERVICE_H
 #define __CONNMAN_SERVICE_H
 
-#include <connman/types.h>
+#include <stdbool.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -39,13 +39,14 @@ enum connman_service_type {
 	CONNMAN_SERVICE_TYPE_SYSTEM    = 1,
 	CONNMAN_SERVICE_TYPE_ETHERNET  = 2,
 	CONNMAN_SERVICE_TYPE_WIFI      = 3,
-	CONNMAN_SERVICE_TYPE_WIMAX     = 4,
-	CONNMAN_SERVICE_TYPE_BLUETOOTH = 5,
-	CONNMAN_SERVICE_TYPE_CELLULAR  = 6,
-	CONNMAN_SERVICE_TYPE_GPS       = 7,
-	CONNMAN_SERVICE_TYPE_VPN       = 8,
-	CONNMAN_SERVICE_TYPE_GADGET    = 9,
+	CONNMAN_SERVICE_TYPE_BLUETOOTH = 4,
+	CONNMAN_SERVICE_TYPE_CELLULAR  = 5,
+	CONNMAN_SERVICE_TYPE_GPS       = 6,
+	CONNMAN_SERVICE_TYPE_VPN       = 7,
+	CONNMAN_SERVICE_TYPE_GADGET    = 8,
 };
+#define	MAX_CONNMAN_SERVICE_TYPES        9
+
 
 enum connman_service_security {
 	CONNMAN_SERVICE_SECURITY_UNKNOWN = 0,
@@ -116,7 +117,7 @@ char **connman_service_get_proxy_servers(struct connman_service *service);
 char **connman_service_get_proxy_excludes(struct connman_service *service);
 const char *connman_service_get_proxy_url(struct connman_service *service);
 const char *connman_service_get_proxy_autoconfig(struct connman_service *service);
-connman_bool_t connman_service_get_favorite(struct connman_service *service);
+bool connman_service_get_favorite(struct connman_service *service);
 
 struct connman_service *connman_service_lookup_from_network(struct connman_network *network);
 
