@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -39,6 +39,9 @@
 #include <connman/inet.h>
 #include <connman/rtnl.h>
 #include <connman/log.h>
+#include <connman/setting.h>
+
+static bool eth_tethering = false;
 
 struct ethernet_data {
 	int index;
@@ -63,6 +66,8 @@ static int cable_connect(struct connman_network *network)
 {
 	DBG("network %p", network);
 
+	connman_network_set_connected(network, true);
+
 	return 0;
 }
 
@@ -70,6 +75,8 @@ static int cable_disconnect(struct connman_network *network)
 {
 	DBG("network %p", network);
 
+	connman_network_set_connected(network, false);
+
 	return 0;
 }
 
@@ -90,7 +97,7 @@ static void add_network(struct connman_device *device,
 
 	network = connman_network_create("carrier",
 					CONNMAN_NETWORK_TYPE_ETHERNET);
-	if (network == NULL)
+	if (!network)
 		return;
 
 	index = connman_device_get_index(device);
@@ -103,11 +110,13 @@ static void add_network(struct connman_device *device,
 		return;
 	}
 
-	connman_network_set_available(network, TRUE);
-
-	connman_network_set_group(network, "cable");
-
-	connman_network_set_connected(network, TRUE);
+	if (!eth_tethering)
+		/*
+		 * Prevent service from starting the reconnect
+		 * procedure as we do not want the DHCP client
+		 * to run when tethering.
+		 */
+		connman_network_set_group(network, "cable");
 
 	ethernet->network = network;
 }
@@ -115,7 +124,7 @@ static void add_network(struct connman_device *device,
 static void remove_network(struct connman_device *device,
 				struct ethernet_data *ethernet)
 {
-	if (ethernet->network == NULL)
+	if (!ethernet->network)
 		return;
 
 	connman_device_remove_network(device, ethernet->network);
@@ -134,10 +143,10 @@ static void ethernet_newlink(unsigned flags, unsigned change, void *user_data)
 	if ((ethernet->flags & IFF_UP) != (flags & IFF_UP)) {
 		if (flags & IFF_UP) {
 			DBG("power on");
-			connman_device_set_powered(device, TRUE);
+			connman_device_set_powered(device, true);
 		} else {
 			DBG("power off");
-			connman_device_set_powered(device, FALSE);
+			connman_device_set_powered(device, false);
 		}
 	}
 
@@ -161,7 +170,7 @@ static int ethernet_probe(struct connman_device *device)
 	DBG("device %p", device);
 
 	ethernet = g_try_new0(struct ethernet_data, 1);
-	if (ethernet == NULL)
+	if (!ethernet)
 		return -ENOMEM;
 
 	connman_device_set_data(device, ethernet);
@@ -217,6 +226,35 @@ static struct connman_device_driver ethernet_driver = {
 	.disable	= ethernet_disable,
 };
 
+static int gadget_probe(struct connman_device *device)
+{
+	DBG("device %p", device);
+	return 0;
+}
+static void gadget_remove(struct connman_device *device)
+{
+	DBG("device %p", device);
+}
+static int gadget_enable(struct connman_device *device)
+{
+	DBG("device %p", device);
+	return 0;
+}
+static int gadget_disable(struct connman_device *device)
+{
+	DBG("device %p", device);
+	return 0;
+}
+
+static struct connman_device_driver gadget_driver = {
+	.name		= "gadget",
+	.type		= CONNMAN_DEVICE_TYPE_GADGET,
+	.probe		= gadget_probe,
+	.remove		= gadget_remove,
+	.enable		= gadget_enable,
+	.disable	= gadget_disable,
+};
+
 static GList *cdc_interface_list = NULL;
 
 static void tech_add_interface(struct connman_technology *technology,
@@ -224,8 +262,7 @@ static void tech_add_interface(struct connman_technology *technology,
 {
 	DBG("index %d name %s ident %s", index, name, ident);
 
-	if (g_list_find(cdc_interface_list,
-			GINT_TO_POINTER((int) index)) != NULL)
+	if (g_list_find(cdc_interface_list, GINT_TO_POINTER((int)index)))
 		return;
 
 	cdc_interface_list = g_list_prepend(cdc_interface_list,
@@ -249,7 +286,7 @@ static void enable_tethering(struct connman_technology *technology,
 	for (list = cdc_interface_list; list; list = list->next) {
 		int index = GPOINTER_TO_INT(list->data);
 
-		connman_technology_tethering_notify(technology, TRUE);
+		connman_technology_tethering_notify(technology, true);
 
 		connman_inet_ifup(index);
 
@@ -269,13 +306,13 @@ static void disable_tethering(struct connman_technology *technology,
 
 		connman_inet_ifdown(index);
 
-		connman_technology_tethering_notify(technology, FALSE);
+		connman_technology_tethering_notify(technology, false);
 	}
 }
 
 static int tech_set_tethering(struct connman_technology *technology,
 				const char *identifier, const char *passphrase,
-				const char *bridge, connman_bool_t enabled)
+				const char *bridge, bool enabled)
 {
 	DBG("bridge %s enabled %d", bridge, enabled);
 
@@ -305,14 +342,125 @@ static struct connman_technology_driver tech_driver = {
 	.probe			= tech_probe,
 	.remove			= tech_remove,
 	.add_interface		= tech_add_interface,
-	.remove_interface 	= tech_remove_interface,
+	.remove_interface	= tech_remove_interface,
 	.set_tethering		= tech_set_tethering,
 };
 
+static int eth_probe(struct connman_technology *technology)
+{
+	return 0;
+}
+
+static void eth_remove(struct connman_technology *technology)
+{
+	DBG("");
+}
+
+static GList *eth_interface_list = NULL;
+
+static void eth_add_interface(struct connman_technology *technology,
+			int index, const char *name, const char *ident)
+{
+	DBG("index %d name %s ident %s", index, name, ident);
+
+	if (g_list_find(eth_interface_list, GINT_TO_POINTER((int)index)))
+		return;
+
+	eth_interface_list = g_list_prepend(eth_interface_list,
+					(GINT_TO_POINTER((int) index)));
+}
+
+static void eth_remove_interface(struct connman_technology *technology,
+								int index)
+{
+	DBG("index %d", index);
+
+	eth_interface_list = g_list_remove(eth_interface_list,
+					GINT_TO_POINTER((int) index));
+}
+
+static void eth_enable_tethering(struct connman_technology *technology,
+						const char *bridge)
+{
+	GList *list;
+
+	for (list = eth_interface_list; list; list = list->next) {
+		int index = GPOINTER_TO_INT(list->data);
+		struct connman_device *device =
+			connman_device_find_by_index(index);
+
+		if (device)
+			connman_device_disconnect_service(device);
+
+		connman_technology_tethering_notify(technology, true);
+
+		connman_inet_ifup(index);
+
+		connman_inet_add_to_bridge(index, bridge);
+
+		eth_tethering = true;
+	}
+}
+
+static void eth_disable_tethering(struct connman_technology *technology,
+						const char *bridge)
+{
+	GList *list;
+
+	for (list = eth_interface_list; list; list = list->next) {
+		int index = GPOINTER_TO_INT(list->data);
+		struct connman_device *device =
+			connman_device_find_by_index(index);
+
+		connman_inet_remove_from_bridge(index, bridge);
+
+		connman_inet_ifdown(index);
+
+		connman_technology_tethering_notify(technology, false);
+
+		if (device)
+			connman_device_reconnect_service(device);
+
+		eth_tethering = false;
+	}
+}
+
+static int eth_set_tethering(struct connman_technology *technology,
+				const char *identifier, const char *passphrase,
+				const char *bridge, bool enabled)
+{
+	if (!connman_technology_is_tethering_allowed(
+			CONNMAN_SERVICE_TYPE_ETHERNET))
+		return 0;
+
+	DBG("bridge %s enabled %d", bridge, enabled);
+
+	if (enabled)
+		eth_enable_tethering(technology, bridge);
+	else
+		eth_disable_tethering(technology, bridge);
+
+	return 0;
+}
+
+static struct connman_technology_driver eth_driver = {
+	.name			= "ethernet",
+	.type			= CONNMAN_SERVICE_TYPE_ETHERNET,
+	.probe			= eth_probe,
+	.remove			= eth_remove,
+	.add_interface		= eth_add_interface,
+	.remove_interface	= eth_remove_interface,
+	.set_tethering		= eth_set_tethering,
+};
+
 static int ethernet_init(void)
 {
 	int err;
 
+	err = connman_technology_driver_register(&eth_driver);
+	if (err < 0)
+		return err;
+
 	err = connman_network_driver_register(&cable_driver);
 	if (err < 0)
 		return err;
@@ -330,16 +478,28 @@ static int ethernet_init(void)
 		return err;
 	}
 
+	err = connman_device_driver_register(&gadget_driver);
+	if (err < 0) {
+		connman_technology_driver_unregister(&tech_driver);
+		connman_device_driver_unregister(&ethernet_driver);
+		connman_network_driver_unregister(&cable_driver);
+		return err;
+	}
+
 	return 0;
 }
 
 static void ethernet_exit(void)
 {
+	connman_technology_driver_unregister(&eth_driver);
+
 	connman_technology_driver_unregister(&tech_driver);
 
 	connman_network_driver_unregister(&cable_driver);
 
 	connman_device_driver_unregister(&ethernet_driver);
+
+	connman_device_driver_unregister(&gadget_driver);
 }
 
 CONNMAN_PLUGIN_DEFINE(ethernet, "Ethernet interface plugin", VERSION,
