@@ -332,14 +332,11 @@ static int parse_address(const char *address_str, int address_family,
 	}
 
 	gw_str = route[2];
-	if (!gw_str || gw_str[0] == '\0') {
-		err = -EINVAL;
-		goto out;
+	if (gw_str && gw_str[0]) {
+		if ((err = check_family(gw_str, address_family)) < 0)
+			goto out;
 	}
 
-	if ((err = check_family(gw_str, address_family)) < 0)
-		goto out;
-
 	g_free(*address);
 	*address = g_strdup(addr_str);
 
@@ -349,7 +346,10 @@ static int parse_address(const char *address_str, int address_family,
 	g_free(*gateway);
 	*gateway = g_strdup(gw_str);
 
-	DBG("address %s/%s via %s", *address, *netmask, *gateway);
+	if (*gateway)
+		DBG("address %s/%s via %s", *address, *netmask, *gateway);
+	else
+		DBG("address %s/%s", *address, *netmask);
 
 out:
 	g_strfreev(route);
@@ -720,6 +720,8 @@ static int load_config(struct connman_config *config)
 	if (!keyfile)
 		return -EIO;
 
+	g_key_file_set_list_separator(keyfile, ',');
+
 	/* Verify keys validity of the global section */
 	check_keys(keyfile, "global", config_possible_keys);
 
@@ -1129,9 +1131,8 @@ static void provision_service(gpointer key, gpointer value,
 	} else {
 		struct connman_ipaddress *address;
 
-		if (config->ipv6_prefix_length == 0 ||
-					!config->ipv6_gateway) {
-			DBG("IPv6 prefix or gateway missing");
+		if (config->ipv6_prefix_length == 0) {
+			DBG("IPv6 prefix missing");
 			return;
 		}
 
@@ -1175,9 +1176,8 @@ static void provision_service(gpointer key, gpointer value,
 	} else {
 		struct connman_ipaddress *address;
 
-		if (config->ipv4_netmask == 0 ||
-					!config->ipv4_gateway) {
-			DBG("IPv4 netmask or gateway missing");
+		if (!config->ipv4_netmask) {
+			DBG("IPv4 netmask missing");
 			return;
 		}
 
@@ -1504,3 +1504,31 @@ void connman_config_free_entries(struct connman_config_entry **entries)
 	g_free(entries);
 	return;
 }
+
+bool __connman_config_address_provisioned(const char *address,
+					const char *netmask)
+{
+	GHashTableIter iter, siter;
+	gpointer value, key, svalue, skey;
+
+	if (!address || !netmask)
+		return false;
+
+	g_hash_table_iter_init(&iter, config_table);
+
+	while (g_hash_table_iter_next(&iter, &key, &value)) {
+		struct connman_config *config = value;
+
+		g_hash_table_iter_init(&siter, config->service_table);
+		while (g_hash_table_iter_next(&siter, &skey, &svalue)) {
+			struct connman_config_service *service = svalue;
+
+			if (!g_strcmp0(address, service->ipv4_address) &&
+					!g_strcmp0(netmask,
+						service->ipv4_netmask))
+				return true;
+		}
+	}
+
+	return false;
+}
