@@ -32,14 +32,14 @@
 #include "connman.h"
 
 static bool connman_state_idle;
-static DBusMessage *session_mode_pending = NULL;
+static dbus_bool_t sessionmode;
 
 static DBusMessage *get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
-	dbus_bool_t offlinemode, sessionmode;
+	dbus_bool_t offlinemode;
 	const char *str;
 
 	DBG("conn %p", conn);
@@ -60,7 +60,6 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	connman_dbus_dict_append_basic(&dict, "OfflineMode",
 					DBUS_TYPE_BOOLEAN, &offlinemode);
 
-	sessionmode = __connman_session_mode();
 	connman_dbus_dict_append_basic(&dict, "SessionMode",
 					DBUS_TYPE_BOOLEAN,
 					&sessionmode);
@@ -105,23 +104,12 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		__connman_technology_set_offlinemode(offlinemode);
 	} else if (g_str_equal(name, "SessionMode")) {
-		dbus_bool_t sessionmode;
 
 		if (type != DBUS_TYPE_BOOLEAN)
 			return __connman_error_invalid_arguments(msg);
 
 		dbus_message_iter_get_basic(&value, &sessionmode);
 
-		if (session_mode_pending)
-			return __connman_error_in_progress(msg);
-
-		__connman_session_set_mode(sessionmode);
-
-		if (sessionmode && !connman_state_idle) {
-			session_mode_pending = dbus_message_ref(msg);
-			return NULL;
-		}
-
 	} else
 		return __connman_error_invalid_property(msg);
 
@@ -170,17 +158,6 @@ static DBusMessage *remove_provider(DBusConnection *conn,
 
 static DBusConnection *connection = NULL;
 
-static void session_mode_notify(void)
-{
-	DBusMessage *reply;
-
-	reply = g_dbus_create_reply(session_mode_pending, DBUS_TYPE_INVALID);
-	g_dbus_send_message(connection, reply);
-
-	dbus_message_unref(session_mode_pending);
-	session_mode_pending = NULL;
-}
-
 static void idle_state(bool idle)
 {
 
@@ -188,10 +165,8 @@ static void idle_state(bool idle)
 
 	connman_state_idle = idle;
 
-	if (!connman_state_idle || !session_mode_pending)
+	if (!connman_state_idle)
 		return;
-
-	session_mode_notify();
 }
 
 static struct connman_notifier technology_notifier = {
@@ -227,13 +202,6 @@ static DBusMessage *connect_provider(DBusConnection *conn,
 
 	DBG("conn %p", conn);
 
-	if (__connman_session_mode()) {
-		connman_info("Session mode enabled: "
-				"direct provider connect disabled");
-
-		return __connman_error_failed(msg, EINVAL);
-	}
-
 	err = __connman_provider_create_and_connect(msg);
 	if (err < 0)
 		return __connman_error_failed(msg, -err);
@@ -487,9 +455,6 @@ void __connman_manager_cleanup(void)
 	if (!connection)
 		return;
 
-	if (session_mode_pending)
-		dbus_message_unref(session_mode_pending);
-
 	connman_notifier_unregister(&technology_notifier);
 
 	g_dbus_unregister_interface(connection, CONNMAN_MANAGER_PATH,
