@@ -30,7 +30,7 @@
 #include "src/shared/nfacct.h"
 #include "src/shared/util.h"
 
-static struct nfacct_info *nfacct;
+static struct nfacct_info *nfacct = NULL;
 
 struct nfacct_rule {
 	char *name;
@@ -188,13 +188,18 @@ int __connman_nfacct_enable(struct nfacct_context *ctx,
 				connman_nfacct_enable_cb_t cb,
 				void *user_data)
 {
-	struct cb_data *cbd;
+	struct cb_data *cbd = NULL;
 	struct nfacct_rule *rule;
 	GList *list;
 	unsigned int id;
 
 	DBG("");
 
+	if (!nfacct)
+		nfacct = nfacct_new();
+	if (!nfacct)
+		goto err;
+
 	for (list = ctx->rules; list; list = list->next) {
 		rule = list->data;
 
@@ -351,13 +356,15 @@ int __connman_nfacct_flush(connman_nfacct_flush_cb_t cb, void *user_data)
 	struct nfacct_flush *nff = g_new0(struct nfacct_flush, 1);
 	unsigned int id;
 
-	DBG("");
+	DBG("nfacct %p", nfacct);
 
-	cbd->data = nff;
+	if (nfacct) {
+		cbd->data = nff;
 
-	id = nfacct_dump(nfacct, false, nfacct_flush_cb, cbd);
-	if (id > 0)
-		return id;
+		id = nfacct_dump(nfacct, false, nfacct_flush_cb, cbd);
+		if (id > 0)
+			return id;
+	}
 
 	g_free(nff);
 	g_free(cbd);
@@ -365,17 +372,6 @@ int __connman_nfacct_flush(connman_nfacct_flush_cb_t cb, void *user_data)
 	return -ECOMM;
 }
 
-int __connman_nfacct_init(void)
-{
-	DBG("");
-
-	nfacct = nfacct_new();
-	if (!nfacct)
-		return -ENOMEM;
-
-	return 0;
-}
-
 void __connman_nfacct_cleanup(void)
 {
 	nfacct_destroy(nfacct);
