@@ -38,7 +38,6 @@ enum rfkill_type {
 	RFKILL_TYPE_WLAN,
 	RFKILL_TYPE_BLUETOOTH,
 	RFKILL_TYPE_UWB,
-	RFKILL_TYPE_WIMAX,
 	RFKILL_TYPE_WWAN,
 	RFKILL_TYPE_GPS,
 	RFKILL_TYPE_FM,
@@ -67,8 +66,6 @@ static enum connman_service_type convert_type(uint8_t type)
 		return CONNMAN_SERVICE_TYPE_WIFI;
 	case RFKILL_TYPE_BLUETOOTH:
 		return CONNMAN_SERVICE_TYPE_BLUETOOTH;
-	case RFKILL_TYPE_WIMAX:
-		return CONNMAN_SERVICE_TYPE_WIMAX;
 	case RFKILL_TYPE_WWAN:
 		return CONNMAN_SERVICE_TYPE_CELLULAR;
 	}
@@ -83,8 +80,6 @@ static enum rfkill_type convert_service_type(enum connman_service_type type)
 		return RFKILL_TYPE_WLAN;
 	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
 		return RFKILL_TYPE_BLUETOOTH;
-	case CONNMAN_SERVICE_TYPE_WIMAX:
-		return RFKILL_TYPE_WIMAX;
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
 		return RFKILL_TYPE_WWAN;
 	case CONNMAN_SERVICE_TYPE_GPS:
@@ -146,8 +141,7 @@ static GIOStatus rfkill_process(GIOChannel *chan)
 	return status;
 }
 
-static gboolean rfkill_event(GIOChannel *chan,
-				GIOCondition cond, gpointer data)
+static gboolean rfkill_event(GIOChannel *chan, GIOCondition cond, gpointer data)
 {
 	if (cond & (G_IO_NVAL | G_IO_HUP | G_IO_ERR))
 		return FALSE;
@@ -160,12 +154,12 @@ static gboolean rfkill_event(GIOChannel *chan,
 
 static GIOChannel *channel = NULL;
 
-int __connman_rfkill_block(enum connman_service_type type, connman_bool_t block)
+int __connman_rfkill_block(enum connman_service_type type, bool block)
 {
 	uint8_t rfkill_type;
 	struct rfkill_event event;
 	ssize_t len;
-	int fd;
+	int fd, err;
 
 	DBG("type %d block %d", type, block);
 
@@ -183,12 +177,15 @@ int __connman_rfkill_block(enum connman_service_type type, connman_bool_t block)
 	event.soft = block;
 
 	len = write(fd, &event, sizeof(event));
-	if (len < 0)
+	if (len < 0) {
 		connman_error("Failed to change RFKILL state");
+		err = len;
+	} else
+		err = 0;
 
 	close(fd);
 
-	return 0;
+	return err;
 }
 
 int __connman_rfkill_init(void)
@@ -227,7 +224,7 @@ void __connman_rfkill_cleanup(void)
 {
 	DBG("");
 
-	if (channel == NULL)
+	if (!channel)
 		return;
 
 	g_io_channel_shutdown(channel, TRUE, NULL);
