@@ -65,7 +65,6 @@ static guint update_timeout = 0;
 
 struct interface_data {
 	int index;
-	char *name;
 	char *ident;
 	enum connman_service_type service_type;
 	enum connman_device_type device_type;
@@ -78,10 +77,9 @@ static void free_interface(gpointer data)
 	struct interface_data *interface = data;
 
 	__connman_technology_remove_interface(interface->service_type,
-			interface->index, interface->name, interface->ident);
+			interface->index, interface->ident);
 
 	g_free(interface->ident);
-	g_free(interface->name);
 	g_free(interface);
 }
 
@@ -106,7 +104,7 @@ static bool wext_interface(char *ifname)
 		return false;
 
 	memset(&wrq, 0, sizeof(wrq));
-	strncpy(wrq.ifr_name, ifname, IFNAMSIZ);
+	strncpy(wrq.ifr_name, ifname, sizeof(wrq.ifr_name) - 1);
 
 	err = ioctl(fd, SIOCGIWNAME, &wrq);
 
@@ -120,11 +118,13 @@ static bool wext_interface(char *ifname)
 
 static void read_uevent(struct interface_data *interface)
 {
-	char *filename, line[128];
+	char *filename, *name, line[128];
 	bool found_devtype;
 	FILE *f;
 
-	if (ether_blacklisted(interface->name)) {
+	name = connman_inet_ifname(interface->index);
+
+	if (ether_blacklisted(name)) {
 		interface->service_type = CONNMAN_SERVICE_TYPE_UNKNOWN;
 		interface->device_type = CONNMAN_DEVICE_TYPE_UNKNOWN;
 	} else {
@@ -132,15 +132,17 @@ static void read_uevent(struct interface_data *interface)
 		interface->device_type = CONNMAN_DEVICE_TYPE_ETHERNET;
 	}
 
-	filename = g_strdup_printf("/sys/class/net/%s/uevent",
-						interface->name);
+	filename = g_strdup_printf("/sys/class/net/%s/uevent", name);
 
 	f = fopen(filename, "re");
 
 	g_free(filename);
 
-	if (!f)
-		return;
+	if (!f) {
+		interface->service_type = CONNMAN_SERVICE_TYPE_UNKNOWN;
+		interface->device_type = CONNMAN_DEVICE_TYPE_UNKNOWN;
+		goto out;
+	}
 
 	found_devtype = false;
 	while (fgets(line, sizeof(line), f)) {
@@ -178,16 +180,18 @@ static void read_uevent(struct interface_data *interface)
 	fclose(f);
 
 	if (found_devtype)
-		return;
+		goto out;
 
 	/* We haven't got a DEVTYPE, let's check if it's a wireless device */
-	if (wext_interface(interface->name)) {
+	if (wext_interface(name)) {
 		interface->service_type = CONNMAN_SERVICE_TYPE_WIFI;
 		interface->device_type = CONNMAN_DEVICE_TYPE_WIFI;
 
-		connman_error("%s runs an unsupported 802.11 driver",
-				interface->name);
+		connman_error("%s runs an unsupported 802.11 driver", name);
 	}
+
+out:
+	g_free(name);
 }
 
 enum connman_device_type __connman_rtnl_get_device_type(int index)
@@ -451,7 +455,6 @@ static void process_newlink(unsigned short type, int index, unsigned flags,
 	if (!interface) {
 		interface = g_new0(struct interface_data, 1);
 		interface->index = index;
-		interface->name = g_strdup(ifname);
 		interface->ident = g_strdup(ident);
 
 		g_hash_table_insert(interface_list,
@@ -477,7 +480,7 @@ static void process_newlink(unsigned short type, int index, unsigned flags,
 	 */
 	if (interface)
 		__connman_technology_add_interface(interface->service_type,
-			interface->index, interface->name, interface->ident);
+			interface->index, interface->ident);
 
 	for (list = watch_list; list; list = list->next) {
 		struct watch_data *watch = list->data;
@@ -580,18 +583,17 @@ static void extract_ipv6_addr(struct ifaddrmsg *msg, int bytes,
 static void process_newaddr(unsigned char family, unsigned char prefixlen,
 				int index, struct ifaddrmsg *msg, int bytes)
 {
+	struct in_addr ipv4_addr = { INADDR_ANY };
+	struct in6_addr ipv6_address, ipv6_local;
 	const char *label = NULL;
 	void *src;
 	char ip_string[INET6_ADDRSTRLEN];
 
 	if (family == AF_INET) {
-		struct in_addr ipv4_addr = { INADDR_ANY };
 
 		extract_ipv4_addr(msg, bytes, &label, &ipv4_addr, NULL, NULL);
 		src = &ipv4_addr;
 	} else if (family == AF_INET6) {
-		struct in6_addr ipv6_address, ipv6_local;
-
 		extract_ipv6_addr(msg, bytes, &ipv6_address, &ipv6_local);
 		if (IN6_IS_ADDR_LINKLOCAL(&ipv6_address))
 			return;
@@ -621,18 +623,16 @@ static void process_newaddr(unsigned char family, unsigned char prefixlen,
 static void process_deladdr(unsigned char family, unsigned char prefixlen,
 				int index, struct ifaddrmsg *msg, int bytes)
 {
+	struct in_addr ipv4_addr = { INADDR_ANY };
+	struct in6_addr ipv6_address, ipv6_local;
 	const char *label = NULL;
 	void *src;
 	char ip_string[INET6_ADDRSTRLEN];
 
 	if (family == AF_INET) {
-		struct in_addr ipv4_addr = { INADDR_ANY };
-
 		extract_ipv4_addr(msg, bytes, &label, &ipv4_addr, NULL, NULL);
 		src = &ipv4_addr;
 	} else if (family == AF_INET6) {
-		struct in6_addr ipv6_address, ipv6_local;
-
 		extract_ipv6_addr(msg, bytes, &ipv6_address, &ipv6_local);
 		if (IN6_IS_ADDR_LINKLOCAL(&ipv6_address))
 			return;
