@@ -211,10 +211,13 @@ static GSList *read_route_dict(GSList *routes, DBusMessageIter *dicts)
 		type = dbus_message_iter_get_arg_type(&value);
 
 		switch (type) {
-		case DBUS_TYPE_STRING:
+		case DBUS_TYPE_INT32:
 			if (g_str_equal(key, "ProtocolFamily"))
 				dbus_message_iter_get_basic(&value, &family);
-			else if (g_str_equal(key, "Network"))
+			break;
+
+		case DBUS_TYPE_STRING:
+			if (g_str_equal(key, "Network"))
 				dbus_message_iter_get_basic(&value, &network);
 			else if (g_str_equal(key, "Netmask"))
 				dbus_message_iter_get_basic(&value, &netmask);
@@ -1075,7 +1078,9 @@ int __vpn_provider_connect(struct vpn_provider *provider, DBusMessage *msg)
 
 	if (provider->driver && provider->driver->connect) {
 		dbus_message_ref(msg);
-		err = provider->driver->connect(provider, connect_cb, msg);
+		err = provider->driver->connect(provider, connect_cb,
+						dbus_message_get_sender(msg),
+						msg);
 	} else
 		return -EOPNOTSUPP;
 
@@ -2557,7 +2562,7 @@ static struct connman_agent_driver agent_driver = {
 	.remove		= agent_remove,
 };
 
-static void remove_unprovisioned_providers()
+static void remove_unprovisioned_providers(void)
 {
 	gchar **providers;
 	GKeyFile *keyfile, *configkeyfile;
@@ -2646,12 +2651,12 @@ void __vpn_provider_cleanup(void)
 {
 	DBG("");
 
+	connman_agent_driver_unregister(&agent_driver);
+
 	g_hash_table_foreach(provider_hash, clean_provider, NULL);
 
 	g_hash_table_destroy(provider_hash);
 	provider_hash = NULL;
 
-	connman_agent_driver_unregister(&agent_driver);
-
 	dbus_connection_unref(connection);
 }
